-- Credits: phyber

Fence_Sort = Fence:NewModule("Sort")
local _G = getfenv(0)
local L = AceLibrary("AceLocale-2.2"):new("Fence_Sort")
L:RegisterTranslations("enUS", function() return {
	["Sort"] = true,
	["Sort options"] = true,
	["Toggles Sort module."] = true,
} end)

L:RegisterTranslations("deDE", function() return {
	["Sort"] = "Sort",
	["Sort options"] = "Sortierungsoptionen",
	["Toggles Sort module."] = "Schaltet Sortierungsmodul ein/aus.)",
} end)

function Fence_Sort:OnInitialize()
	self.db = Fence:AcquireDBNamespace("Sort")
	
	-- options table
	Fence.options.args.sort = {
		type = 'group',
		name = L["Sort"],
		desc = L["Sort options"],
		args = {
			toggle = {
				type = 'toggle',
				name = L["Sort"],
				desc = L["Toggles Sort module."],
				get = function() return Fence:IsModuleActive("Sort") end,
				set = function(v) Fence:ToggleModuleActive("Sort", v) end
			}
		}
	}
end

function Fence_Sort:OnEnable()
	self:RegisterEvent("AH_LOADED")
end

function Fence_Sort:OnDisable()
	if self:IsHooked(BrowseCurrentBidSort, "SetWidth") then
		self:Unhook(BrowseCurrentBidSort, "SetWidth") end
end


function Fence_Sort:AH_LOADED()
	self:UnregisterEvent("AH_LOADED")
	self:SecureHook(BrowseCurrentBidSort, "SetWidth", "NewSetWidth")
	-- We need this to prevent some weird shrinkage later on.
	self.BQSWidth = BrowseQualitySort:GetWidth()

	-- Make the buttons.
	local BrowseBuyoutSort = CreateFrame("Button", "BrowseBuyoutSort", AuctionFrameBrowse, "AuctionSortButtonTemplate")
	BrowseBuyoutSort:SetWidth(95)
	BrowseBuyoutSort:SetHeight(19)
	BrowseBuyoutSort:SetText(BUYOUT_PRICE)
	BrowseBuyoutSort:SetScript("OnClick", function()
		SortAuctionItems("list", "buyout")
		self:UpdateArrow(BrowseBuyoutSort, "list", "buyout")
	end)

	local BrowseNameSort = CreateFrame("Button", "BrowseNameSort", AuctionFrameBrowse, "AuctionSortButtonTemplate")
	BrowseNameSort:SetWidth(95)
	BrowseNameSort:SetHeight(19)
	BrowseNameSort:SetText(NAME)
	BrowseNameSort:SetScript("OnClick", function()
		SortAuctionItems("list", "name")
		self:UpdateArrow(BrowseNameSort, "list", "name")
	end)

	BrowseBuyoutSort:ClearAllPoints()
	BrowseBuyoutSort:SetPoint("LEFT", "BrowseCurrentBidSort", "RIGHT", -2, 0)
	BrowseBuyoutSort:Show()

	BrowseNameSort:ClearAllPoints()
	BrowseNameSort:SetPoint("TOPLEFT", "AuctionFrameBrowse", "TOPLEFT", 186, -82)
	BrowseNameSort:Show()

	BrowseQualitySort:ClearAllPoints()
	BrowseQualitySort:SetPoint("LEFT", "BrowseNameSort", "RIGHT", -2, 0)
	BrowseQualitySort:SetWidth(self.BQSWidth)
	BrowseQualitySort:SetWidth(BrowseQualitySort:GetWidth() - BrowseNameSort:GetWidth())
	BrowseQualitySort:Show()

	self:NewSetWidth(nil, 207)
end

function Fence_Sort:NewSetWidth(obj, width)
	if width >= 180 then
		width = width - BrowseBuyoutSort:GetWidth() + 2
	end

	-- Unhook before we set the new width, we don't want to create a pime taradox.
	if self:IsHooked(BrowseCurrentBidSort, "SetWidth") then
		self:Unhook(BrowseCurrentBidSort, "SetWidth")
	end

	-- Set the new width
	BrowseCurrentBidSort:SetWidth(width)

	-- Hook it again
	if not self:IsHooked(BrowseCurrentBidSort, "SetWidth") then
		self:SecureHook(BrowseCurrentBidSort, "SetWidth", "NewSetWidth")
	end
end

function Fence_Sort:UpdateArrow(button, type, sort)
	if not IsAuctionSortReversed(type, sort) then
		_G[button:GetName().."Arrow"]:SetTexCoord(0, 0.5625, 1.0, 0)
	else
		_G[button:GetName().."Arrow"]:SetTexCoord(0, 0.5625, 0, 1.0)
	end
end
